@@ -1,5 +1,4 @@
-#include <gtk/gtkprintunixdialog.h>
-#include <gtk/gtkpagesetupunixdialog.h>
+#include <gtk/gtkunixprint.h>
 #include <gdk/gdkkeysyms.h>
 #include <X11/Xatom.h>
 #include <gdkx.h>
@@ -17,7 +16,7 @@ static Window
 find_toplevel_window (Window xid)
 {
   Window root, parent, *children;
-  int nchildren;
+  guint nchildren;
 
   do
     {
@@ -90,7 +89,7 @@ static void
 realize_callback (GtkWidget  *widget,
 		  WidgetInfo *info)
 {
-  g_timeout_add (500, (GSourceFunc)adjust_size_callback, info);
+  gdk_threads_add_timeout (500, (GSourceFunc)adjust_size_callback, info);
 }
 
 static WidgetInfo *
@@ -911,6 +910,19 @@ create_image (void)
   return new_widget_info ("image", vbox, SMALL);
 }
 
+static WidgetInfo *
+create_volume_button (void)
+{
+  GtkWidget *button, *widget;
+
+  button = gtk_volume_button_new ();
+  gtk_scale_button_set_value (GTK_SCALE_BUTTON (button), 33);
+  /* Hack: get the private dock */
+  widget = GTK_SCALE_BUTTON (button)->plus_button->parent->parent->parent;
+  gtk_widget_show_all (widget);
+  return new_widget_info ("volumebutton", widget, ASIS);
+}
+
 static WidgetInfo *
 create_assistant (void)
 {
@@ -981,6 +993,7 @@ get_all_widgets (void)
   retval = g_list_prepend (retval, create_recent_chooser_dialog ());
   retval = g_list_prepend (retval, create_page_setup_dialog ());
   retval = g_list_prepend (retval, create_print_dialog ());
+  retval = g_list_prepend (retval, create_volume_button ());
 
   return retval;
 }
