@@ -103,16 +103,12 @@ fill_pcx_context(struct pcx_context *context)
 static void
 free_pcx_context(struct pcx_context *context, gboolean unref_pixbuf)
 {
-	if(context->header)
-		g_free(context->header);
-	if(context->buf)
-		g_free(context->buf);
+	g_free(context->header);
+	g_free(context->buf);
 	if(unref_pixbuf && context->pixbuf)
 		g_object_unref(context->pixbuf);
-	if(context->line)
-		g_free(context->line);
-	if(context->p_data)
-		g_free(context->p_data);
+	g_free(context->line);
+	g_free(context->p_data);
 
 	g_free(context);
 }
@@ -236,14 +232,12 @@ pcx_chop_context_buf(struct pcx_context *context, guint size)
 {
 	guint i, j;
 
-	if(size > context->buf_pos)
+	if (size > context->buf_pos)
 		return FALSE;
-	else if(size < 0)
-		return FALSE;
-	else if(size == 0)
+	else if (size == 0)
 		return TRUE;
 
-	for(i = 0, j = size; j < context->buf_pos; i++, j++)
+	for (i = 0, j = size; j < context->buf_pos; i++, j++)
 		context->buf[i] = context->buf[j];
 
 	context->buf_pos -= size;
@@ -629,7 +623,7 @@ gdk_pixbuf__pcx_load_increment(gpointer data, const guchar *buf, guint size,
 				case 4:
 				case 8:
 					if(context->num_planes != 1) {
-						g_set_error(error, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_UNKNOWN_TYPE, _("Image has unsupported number of %d-bit planes"), context->bpp);
+					  g_set_error(error, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_UNKNOWN_TYPE, _("Image has unsupported number of %d-bit planes"), (gint)context->bpp);
 						return FALSE;
 					}
 					break;
@@ -725,16 +719,20 @@ gdk_pixbuf__pcx_stop_load(gpointer data, GError **error)
 	return TRUE;
 }
 
-void
-MODULE_ENTRY (pcx, fill_vtable) (GdkPixbufModule *module)
+#ifndef INCLUDE_pcx
+#define MODULE_ENTRY(function) G_MODULE_EXPORT void function
+#else
+#define MODULE_ENTRY(function) void _gdk_pixbuf__pcx_ ## function
+#endif
+
+MODULE_ENTRY (fill_vtable) (GdkPixbufModule *module)
 {
 	module->begin_load = gdk_pixbuf__pcx_begin_load;
 	module->stop_load = gdk_pixbuf__pcx_stop_load;
 	module->load_increment = gdk_pixbuf__pcx_load_increment;
 }
 
-void
-MODULE_ENTRY (pcx, fill_info) (GdkPixbufFormat *info)
+MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
 {
 	static GdkPixbufModulePattern signature[] = {
 		{ "\x0a \x01", NULL, 100 },
