@@ -1037,13 +1037,12 @@ gdk_pixbuf__pnm_image_load_increment (gpointer data,
 }
 
 #ifndef INCLUDE_pnm
-#define MODULE_ENTRY(type,function) function
+#define MODULE_ENTRY(function) G_MODULE_EXPORT void function
 #else
-#define MODULE_ENTRY(type,function) _gdk_pixbuf__ ## type ## _ ## function
+#define MODULE_ENTRY(function) void _gdk_pixbuf__pnm_ ## function
 #endif
 
-void
-MODULE_ENTRY (pnm, fill_vtable) (GdkPixbufModule *module)
+MODULE_ENTRY (fill_vtable) (GdkPixbufModule *module)
 {
 	module->load = gdk_pixbuf__pnm_image_load;
 	module->begin_load = gdk_pixbuf__pnm_image_begin_load;
@@ -1051,8 +1050,7 @@ MODULE_ENTRY (pnm, fill_vtable) (GdkPixbufModule *module)
 	module->load_increment = gdk_pixbuf__pnm_image_load_increment;
 }
 
-void
-MODULE_ENTRY (pnm, fill_info) (GdkPixbufFormat *info)
+MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
 {
 	static GdkPixbufModulePattern signature[] = {
 		{ "P1", NULL, 100 },
