@@ -141,27 +141,14 @@ static void free_buffer (guchar *pixels, gpointer data)
 	g_free (pixels);
 }
 
-#if TIFFLIB_VERSION >= 20031226
-static gboolean tifflibversion (int *major, int *minor, int *revision)
-{
-        if (sscanf (TIFFGetVersion(), 
-                    "LIBTIFF, Version %d.%d.%d", 
-                    major, minor, revision) < 3)
-                return FALSE;
-
-        return TRUE;
-}
-#endif
-
 static GdkPixbuf *
 tiff_image_parse (TIFF *tiff, TiffContext *context, GError **error)
 {
 	guchar *pixels = NULL;
 	gint width, height, rowstride, bytes;
 	GdkPixbuf *pixbuf;
-#if TIFFLIB_VERSION >= 20031226
-        gint major, minor, revision;
-#endif
+	uint16 orientation = 0;
+	uint16 transform = 0;
 
         /* We're called with the lock held. */
         
@@ -212,6 +199,11 @@ tiff_image_parse (TIFF *tiff, TiffContext *context, GError **error)
                 gint h = height;
 		(* context->size_func) (&w, &h, context->user_data);
                 
+		/* This is a signal that this function is being called
+		   to support gdk_pixbuf_get_file_info, so we can stop
+		   parsing the tiff file at this point. It is not an
+		   error condition. */
+
                 if (w == 0 || h == 0)
                     return NULL;
         }
@@ -238,95 +230,73 @@ tiff_image_parse (TIFF *tiff, TiffContext *context, GError **error)
                 return NULL;
         }
 
+	/* Set the "orientation" key associated with this image. libtiff 
+	   orientation handling is odd, so further processing is required
+	   by higher-level functions based on this tag. If the embedded
+	   orientation tag is 1-4, libtiff flips/mirrors the image as
+	   required, and no client processing is required - so we report 
+	   no orientation. Orientations 5-8 require rotations which would 
+	   swap the width and height of the image. libtiff does not do this. 
+	   Instead it interprets orientations 5-8 the same as 1-4. 
+	   See http://bugzilla.remotesensing.org/show_bug.cgi?id=1548.
+	   To correct for this, the client must apply the transform normally
+	   used for orientation 5 to both orientations 5 and 7, and apply
+	   the transform normally used for orientation 7 for both
+	   orientations 6 and 8. Then everythings works out OK! */
+	
+	TIFFGetField (tiff, TIFFTAG_ORIENTATION, &orientation);
+
+	switch (orientation) {
+		case 5:
+		case 7:
+			transform = 5;
+			break;
+		case 6:
+		case 8:
+			transform = 7;
+			break;
+		default:
+			transform = 0;
+			break;
+	}
+
+	if (transform > 0 ) {
+		gchar str[5];
+		g_snprintf (str, sizeof (str), "%d", transform);
+		gdk_pixbuf_set_option (pixbuf, "orientation", str);
+	}
+
 	if (context && context->prepare_func)
 		(* context->prepare_func) (pixbuf, NULL, context->user_data);
 
-#if TIFFLIB_VERSION >= 20031226
-        if (tifflibversion(&major, &minor, &revision) && major == 3 &&
-            (minor > 6 || (minor == 6 && revision > 0))) {                
-                if (!TIFFReadRGBAImageOriented (tiff, width, height, (uint32 *)pixels, ORIENTATION_TOPLEFT, 1) || global_error) 
-                {
-                        tiff_set_error (error,
-                                        GDK_PIXBUF_ERROR_FAILED,
-                                        _("Failed to load RGB data from TIFF file"));
-                        g_object_unref (pixbuf);
-                        return NULL;
-                }
+	if (!TIFFReadRGBAImageOriented (tiff, width, height, (uint32 *)pixels, ORIENTATION_TOPLEFT, 1) || global_error) {
+		tiff_set_error (error,
+                                GDK_PIXBUF_ERROR_FAILED,
+                                _("Failed to load RGB data from TIFF file"));
+		g_object_unref (pixbuf);
+		return NULL;
+	}
 
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-                /* Turns out that the packing used by TIFFRGBAImage depends on 
-                 * the host byte order... 
-                 */ 
-                while (pixels < pixbuf->pixels + bytes) {
-                        uint32 pixel = *(uint32 *)pixels;
-                        int r = TIFFGetR(pixel);
-                        int g = TIFFGetG(pixel);
-                        int b = TIFFGetB(pixel);
-                        int a = TIFFGetA(pixel);
-                        *pixels++ = r;
-                        *pixels++ = g;
-                        *pixels++ = b;
-                        *pixels++ = a;
-                }
+	/* Turns out that the packing used by TIFFRGBAImage depends on 
+         * the host byte order... 
+         */ 
+	while (pixels < pixbuf->pixels + bytes) {
+		uint32 pixel = *(uint32 *)pixels;
+		int r = TIFFGetR(pixel);
+		int g = TIFFGetG(pixel);
+		int b = TIFFGetB(pixel);
+		int a = TIFFGetA(pixel);
+		*pixels++ = r;
+		*pixels++ = g;
+		*pixels++ = b;
+		*pixels++ = a;
+	}
 #endif
-        }
-        else 
-#endif
-              {
-                uint32 *rast, *tmp_rast;
-                gint x, y;
-                guchar *tmppix;
-
-                /* Yes, it needs to be _TIFFMalloc... */
-                rast = (uint32 *) _TIFFmalloc (width * height * sizeof (uint32));
-                if (!rast) {
-                        g_set_error (error,
-                                     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-                                     _("Insufficient memory to open TIFF file"));
-                        g_object_unref (pixbuf);
-                        
-                        return NULL;
-                }
-                if (!TIFFReadRGBAImage (tiff, width, height, rast, 1) || global_error) {
-                        tiff_set_error (error,
-                                        GDK_PIXBUF_ERROR_FAILED,
-                                        _("Failed to load RGB data from TIFF file"));
-                        g_object_unref (pixbuf);
-                        _TIFFfree (rast);
-                        
-                        return NULL;
-                }
-                
-                pixels = gdk_pixbuf_get_pixels (pixbuf);
-                
-                g_assert (pixels);
-                
-                tmppix = pixels;
-                
-                for (y = 0; y < height; y++) {
-                        /* Unexplainable...are tiffs backwards? */
-                        /* Also looking at the GIMP plugin, this
-                         * whole reading thing can be a bit more
-                         * robust.
-                         */
-                        tmp_rast = rast + ((height - y - 1) * width);
-                        for (x = 0; x < width; x++) {
-                                tmppix[0] = TIFFGetR (*tmp_rast);
-                                tmppix[1] = TIFFGetG (*tmp_rast);
-                                tmppix[2] = TIFFGetB (*tmp_rast);
-                                tmppix[3] = TIFFGetA (*tmp_rast);
-                                tmp_rast++;
-                                tmppix += 4;
-                        }
-                }
-                
-                _TIFFfree (rast);
-             }
 
 	if (context && context->update_func)
 		(* context->update_func) (pixbuf, 0, 0, width, height, context->user_data);
-        
+
         return pixbuf;
 }
 
@@ -405,7 +375,7 @@ gdk_pixbuf__tiff_image_begin_load (GdkPixbufModuleSizeFunc size_func,
 }
 
 static tsize_t
-tiff_read (thandle_t handle, tdata_t buf, tsize_t size)
+tiff_load_read (thandle_t handle, tdata_t buf, tsize_t size)
 {
         TiffContext *context = (TiffContext *)handle;
         
@@ -418,19 +388,19 @@ tiff_read (thandle_t handle, tdata_t buf, tsize_t size)
 }
 
 static tsize_t
-tiff_write (thandle_t handle, tdata_t buf, tsize_t size)
+tiff_load_write (thandle_t handle, tdata_t buf, tsize_t size)
 {
         return -1;
 }
 
 static toff_t
-tiff_seek (thandle_t handle, toff_t offset, int whence)
+tiff_load_seek (thandle_t handle, toff_t offset, int whence)
 {
         TiffContext *context = (TiffContext *)handle;
         
         switch (whence) {
         case SEEK_SET:
-                if (offset > context->used || offset < 0)
+                if (offset > context->used)
                         return -1;
                 context->pos = offset;
                 break;
@@ -446,19 +416,18 @@ tiff_seek (thandle_t handle, toff_t offset, int whence)
                 break;
         default:
                 return -1;
-                break;
         }
         return context->pos;
 }
 
 static int
-tiff_close (thandle_t context)
+tiff_load_close (thandle_t context)
 {
         return 0;
 }
 
 static toff_t
-tiff_size (thandle_t handle)
+tiff_load_size (thandle_t handle)
 {
         TiffContext *context = (TiffContext *)handle;
         
@@ -466,7 +435,7 @@ tiff_size (thandle_t handle)
 }
 
 static int
-tiff_map_file (thandle_t handle, tdata_t *buf, toff_t *size)
+tiff_load_map_file (thandle_t handle, tdata_t *buf, toff_t *size)
 {
         TiffContext *context = (TiffContext *)handle;
         
@@ -477,7 +446,7 @@ tiff_map_file (thandle_t handle, tdata_t *buf, toff_t *size)
 }
 
 static void
-tiff_unmap_file (thandle_t handle, tdata_t data, toff_t offset)
+tiff_load_unmap_file (thandle_t handle, tdata_t data, toff_t offset)
 {
 }
 
@@ -494,10 +463,10 @@ gdk_pixbuf__tiff_image_stop_load (gpointer data,
         tiff_push_handlers ();
         
         tiff = TIFFClientOpen ("libtiff-pixbuf", "r", data, 
-                               tiff_read, tiff_write, 
-                               tiff_seek, tiff_close, 
-                               tiff_size, 
-                               tiff_map_file, tiff_unmap_file);
+                               tiff_load_read, tiff_load_write, 
+                               tiff_load_seek, tiff_load_close, 
+                               tiff_load_size, 
+                               tiff_load_map_file, tiff_load_unmap_file);
         if (!tiff || global_error) {
                 tiff_set_error (error,
                                 GDK_PIXBUF_ERROR_FAILED,
@@ -578,17 +547,252 @@ gdk_pixbuf__tiff_image_load_increment (gpointer data, const guchar *buf,
 	return TRUE;
 }
 
-void
-MODULE_ENTRY (tiff, fill_vtable) (GdkPixbufModule *module)
+typedef struct {
+        gchar *buffer;
+        guint allocated;
+        guint used;
+        guint pos;
+} TiffSaveContext;
+
+static tsize_t
+tiff_save_read (thandle_t handle, tdata_t buf, tsize_t size)
+{
+        return -1;
+}
+
+static tsize_t
+tiff_save_write (thandle_t handle, tdata_t buf, tsize_t size)
+{
+        TiffSaveContext *context = (TiffSaveContext *)handle;
+
+        /* Modify buffer length */
+        if (context->pos + size > context->used)
+                context->used = context->pos + size;
+
+        /* Realloc */
+        if (context->used > context->allocated) {
+                context->buffer = g_realloc (context->buffer, context->pos + size);
+                context->allocated = context->used;
+        }
+
+        /* Now copy the data */
+        memcpy (context->buffer + context->pos, buf, size);
+
+        /* Update pos */
+        context->pos += size;
+
+        return size;
+}
+
+static toff_t
+tiff_save_seek (thandle_t handle, toff_t offset, int whence)
+{
+        TiffSaveContext *context = (TiffSaveContext *)handle;
+
+        switch (whence) {
+        case SEEK_SET:
+                context->pos = offset;
+                break;
+        case SEEK_CUR:
+                context->pos += offset;
+                break;
+        case SEEK_END:
+                context->pos = context->used + offset;
+                break;
+        default:
+                return -1;
+        }
+        return context->pos;
+}
+
+static int
+tiff_save_close (thandle_t context)
+{
+        return 0;
+}
+
+static toff_t
+tiff_save_size (thandle_t handle)
+{
+        return -1;
+}
+
+static TiffSaveContext *
+create_save_context (void)
+{
+        TiffSaveContext *context;
+
+        context = g_new (TiffSaveContext, 1);
+        context->buffer = NULL;
+        context->allocated = 0;
+        context->used = 0;
+        context->pos = 0;
+
+        return context;
+}
+
+static void
+free_save_context (TiffSaveContext *context)
+{
+        g_free (context->buffer);
+        g_free (context);
+}
+
+static gboolean
+gdk_pixbuf__tiff_image_save_to_callback (GdkPixbufSaveFunc   save_func,
+                                         gpointer            user_data,
+                                         GdkPixbuf          *pixbuf, 
+                                         gchar             **keys,
+                                         gchar             **values,
+                                         GError            **error)
+{
+        TIFF *tiff;
+        gint width, height, rowstride;
+        guchar *pixels;
+        gboolean has_alpha;
+        gushort alpha_samples[1] = { EXTRASAMPLE_UNASSALPHA };
+        int y;
+        TiffSaveContext *context;
+        gboolean retval;
+
+        tiff_push_handlers ();
+
+        context = create_save_context ();
+        tiff = TIFFClientOpen ("libtiff-pixbuf", "w", context,  
+                               tiff_save_read, tiff_save_write, 
+                               tiff_save_seek, tiff_save_close, 
+                               tiff_save_size, 
+                               NULL, NULL);
+
+        if (!tiff || global_error) {
+                tiff_set_error (error,
+                                GDK_PIXBUF_ERROR_FAILED,
+                                _("Failed to save TIFF image"));
+
+                tiff_pop_handlers ();
+
+                free_save_context (context);
+                return FALSE;
+        }
+
+        rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+        pixels = gdk_pixbuf_get_pixels (pixbuf);
+
+        has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
+
+        height = gdk_pixbuf_get_height (pixbuf);
+        width = gdk_pixbuf_get_width (pixbuf);
+
+        TIFFSetField (tiff, TIFFTAG_IMAGEWIDTH, width);
+        TIFFSetField (tiff, TIFFTAG_IMAGELENGTH, height);
+        TIFFSetField (tiff, TIFFTAG_BITSPERSAMPLE, 8);
+        TIFFSetField (tiff, TIFFTAG_SAMPLESPERPIXEL, has_alpha ? 4 : 3);
+        TIFFSetField (tiff, TIFFTAG_ROWSPERSTRIP, height);
+
+        if (has_alpha)
+                TIFFSetField (tiff, TIFFTAG_EXTRASAMPLES, 1, alpha_samples);
+
+        TIFFSetField (tiff, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
+        TIFFSetField (tiff, TIFFTAG_FILLORDER, FILLORDER_MSB2LSB);        
+        TIFFSetField (tiff, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
+
+        for (y = 0; y < height; y++) {
+                if (TIFFWriteScanline (tiff, pixels + y * rowstride, y, 0) == -1 ||
+                    global_error)
+                        break;
+        }
+
+        if (global_error) {
+                tiff_set_error (error,
+                                GDK_PIXBUF_ERROR_FAILED,
+                                _("Failed to write TIFF data"));
+
+                TIFFClose (tiff);
+
+                free_save_context (context);               
+                tiff_pop_handlers ();
+                
+                return FALSE;
+        }
+
+        TIFFClose (tiff);
+        if (global_error) {
+                tiff_set_error (error,
+                                GDK_PIXBUF_ERROR_FAILED,
+                                _("TIFFClose operation failed"));
+
+                free_save_context (context);               
+                tiff_pop_handlers ();
+                
+                return FALSE;
+        }
+
+        tiff_pop_handlers ();
+
+        /* Now call the callback */
+        retval = save_func (context->buffer, context->used, error, user_data);
+
+        free_save_context (context);               
+        
+        return retval;
+}
+
+static gboolean
+save_to_file_cb (const gchar *buf,
+		 gsize count,
+		 GError **error,
+		 gpointer data)
+{
+	gint bytes;
+	
+	while (count > 0) {
+		bytes = fwrite (buf, sizeof (gchar), count, (FILE *) data);
+		if (bytes <= 0)
+			break;
+		count -= bytes;
+		buf += bytes;
+	}
+
+	if (count) {
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+			     GDK_PIXBUF_ERROR_FAILED,
+			     _("Couldn't write to TIFF file"));
+		return FALSE;
+	}
+	
+	return TRUE;
+}
+
+static gboolean
+gdk_pixbuf__tiff_image_save (FILE          *f, 
+                             GdkPixbuf     *pixbuf, 
+                             gchar        **keys,
+                             gchar        **values,
+                             GError       **error)
+{
+	return gdk_pixbuf__tiff_image_save_to_callback (save_to_file_cb,
+                                                        f, pixbuf, keys,
+                                                        values, error);
+}
+
+#ifndef INCLUDE_tiff
+#define MODULE_ENTRY(function) G_MODULE_EXPORT void function
+#else
+#define MODULE_ENTRY(function) void _gdk_pixbuf__tiff_ ## function
+#endif
+
+MODULE_ENTRY (fill_vtable) (GdkPixbufModule *module)
 {
         module->load = gdk_pixbuf__tiff_image_load;
         module->begin_load = gdk_pixbuf__tiff_image_begin_load;
         module->stop_load = gdk_pixbuf__tiff_image_stop_load;
         module->load_increment = gdk_pixbuf__tiff_image_load_increment;
+        module->save = gdk_pixbuf__tiff_image_save;
+        module->save_to_callback = gdk_pixbuf__tiff_image_save_to_callback;
 }
 
-void
-MODULE_ENTRY (tiff, fill_info) (GdkPixbufFormat *info)
+MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
 {
         static GdkPixbufModulePattern signature[] = {
                 { "MM \x2a", "  z ", 100 },
@@ -611,6 +815,6 @@ MODULE_ENTRY (tiff, fill_info) (GdkPixbufFormat *info)
 	info->mime_types = mime_types;
 	info->extensions = extensions;
         /* not threadsafe, due to the error handler handling */
-	info->flags = 0;
+	info->flags = GDK_PIXBUF_FORMAT_WRITABLE;
 	info->license = "LGPL";
 }
