@@ -750,7 +750,7 @@ _gtk_accel_label_class_get_accelerator_label (GtkAccelLabelClass *klass,
 	      
 	      strcpy (msg, "keyboard label|");
 	      g_strlcat (msg, tmp, 128);
-	      str = dgettext (GETTEXT_PACKAGE, msg);
+	      str = _(msg);
 	      if (str == msg)
 		{
 		  g_string_append (gstring, tmp);
