@@ -19,19 +19,24 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* 
+/*
  * Author: James Henstridge <james@daa.com.au>
  *
  * Modified by the GTK+ Team and others 2003.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
+
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_ACTION_GROUP_H__
 #define __GTK_ACTION_GROUP_H__
 
 #include <gtk/gtkaction.h>
-#include <gtk/gtkitemfactory.h> /* for GtkTranslateFunc */
+#include <gtk/gtktypeutils.h> /* for GtkTranslateFunc */
 
 G_BEGIN_DECLS
 
@@ -103,7 +108,7 @@ struct _GtkRadioActionEntry
   gint   value; 
 };
 
-GType           gtk_action_group_get_type                (void);
+GType           gtk_action_group_get_type                (void) G_GNUC_CONST;
 GtkActionGroup *gtk_action_group_new                     (const gchar                *name);
 G_CONST_RETURN gchar *gtk_action_group_get_name          (GtkActionGroup             *action_group);
 gboolean        gtk_action_group_get_sensitive           (GtkActionGroup             *action_group);
