@@ -26,10 +26,9 @@
 
 #include <config.h>
 #include "gtkbin.h"
+#include "gtkintl.h"
+#include "gtkalias.h"
 
-
-static void gtk_bin_class_init  (GtkBinClass    *klass);
-static void gtk_bin_init        (GtkBin         *bin);
 static void gtk_bin_add         (GtkContainer   *container,
 			         GtkWidget      *widget);
 static void gtk_bin_remove      (GtkContainer   *container,
@@ -41,36 +40,7 @@ static void gtk_bin_forall      (GtkContainer   *container,
 static GType gtk_bin_child_type (GtkContainer   *container);
 
 
-static GtkContainerClass *parent_class = NULL;
-
-
-GType
-gtk_bin_get_type (void)
-{
-  static GType bin_type = 0;
-
-  if (!bin_type)
-    {
-      static const GTypeInfo bin_info =
-      {
-	sizeof (GtkBinClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_bin_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkBin),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_bin_init,
-	NULL,		/* value_table */
-      };
-
-      bin_type = g_type_register_static (GTK_TYPE_CONTAINER, "GtkBin", 
-					 &bin_info, G_TYPE_FLAG_ABSTRACT);
-    }
-
-  return bin_type;
-}
+G_DEFINE_ABSTRACT_TYPE (GtkBin, gtk_bin, GTK_TYPE_CONTAINER)
 
 static void
 gtk_bin_class_init (GtkBinClass *class)
@@ -79,8 +49,6 @@ gtk_bin_class_init (GtkBinClass *class)
 
   container_class = (GtkContainerClass*) class;
 
-  parent_class = g_type_class_peek_parent (class);
-
   container_class->add = gtk_bin_add;
   container_class->remove = gtk_bin_remove;
   container_class->forall = gtk_bin_forall;
@@ -111,8 +79,6 @@ gtk_bin_add (GtkContainer *container,
 {
   GtkBin *bin = GTK_BIN (container);
 
-  g_return_if_fail (GTK_IS_WIDGET (child));
-
   if (bin->child != NULL)
     {
       g_warning ("Attempting to add a widget with type %s to a %s, "
@@ -136,7 +102,6 @@ gtk_bin_remove (GtkContainer *container,
   GtkBin *bin = GTK_BIN (container);
   gboolean widget_was_visible;
 
-  g_return_if_fail (GTK_IS_WIDGET (child));
   g_return_if_fail (bin->child == child);
 
   widget_was_visible = GTK_WIDGET_VISIBLE (child);
@@ -159,8 +124,6 @@ gtk_bin_forall (GtkContainer *container,
 {
   GtkBin *bin = GTK_BIN (container);
 
-  g_return_if_fail (callback != NULL);
-
   if (bin->child)
     (* callback) (bin->child, callback_data);
 }
@@ -182,3 +145,6 @@ gtk_bin_get_child (GtkBin *bin)
 
   return bin->child;
 }
+
+#define __GTK_BIN_C__
+#include "gtkaliasdef.c"
