@@ -50,11 +50,6 @@ static void gtk_builder_get_property   (GObject         *object,
                                         GParamSpec      *pspec);
 static GType gtk_builder_real_get_type_from_name (GtkBuilder  *builder,
                                                   const gchar *type_name);
-static gboolean _gtk_builder_enum_from_string (GType         type, 
-					       const gchar  *string,
-					       gint         *enum_value,
-					       GError      **error);
-
 
 enum {
   PROP_0,
@@ -67,7 +62,6 @@ struct _GtkBuilderPrivate
   GHashTable *objects;
   GSList *delayed_properties;
   GSList *signals;
-  GSList *root_objects;
   gchar *filename;
 };
 
@@ -92,7 +86,7 @@ gtk_builder_class_init (GtkBuilderClass *klass)
   * The translation domain used when translating property values that
   * have been marked as translatable in interface descriptions.
   * If the translation domain is %NULL, #GtkBuilder uses gettext(),
-  * otherwise dgettext().
+  * otherwise g_dgettext().
   *
   * Since: 2.12
   */
@@ -114,7 +108,7 @@ gtk_builder_init (GtkBuilder *builder)
                                                GtkBuilderPrivate);
   builder->priv->domain = NULL;
   builder->priv->objects = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                                  g_free, NULL);
+                                                  g_free, g_object_unref);
 }
 
 
@@ -135,9 +129,6 @@ gtk_builder_finalize (GObject *object)
   g_slist_foreach (priv->signals, (GFunc) _free_signal_info, NULL);
   g_slist_free (priv->signals);
   
-  g_slist_foreach (priv->root_objects, (GFunc) g_object_unref, NULL);
-  g_slist_free (priv->root_objects);
-
   G_OBJECT_CLASS (gtk_builder_parent_class)->finalize (object);
 }
 
@@ -279,7 +270,7 @@ gtk_builder_get_parameters (GtkBuilder  *builder,
                                             prop->name);
       if (!pspec)
         {
-          g_warning ("Unknown property: %s.%s\n",
+          g_warning ("Unknown property: %s.%s",
                      g_type_name (object_type), prop->name);
           continue;
         }
@@ -338,7 +329,8 @@ gtk_builder_get_parameters (GtkBuilder  *builder,
 static GObject *
 gtk_builder_get_internal_child (GtkBuilder  *builder,
                                 ObjectInfo  *info,
-                                const gchar *childname)
+                                const gchar *childname,
+				GError      **error)
 {
   GObject *obj = NULL;
 
@@ -363,14 +355,19 @@ gtk_builder_get_internal_child (GtkBuilder  *builder,
     };
 
   if (!obj)
-    g_error ("Unknown internal child: %s\n", childname);
-
+    {
+      g_set_error (error,
+		   GTK_BUILDER_ERROR,
+		   GTK_BUILDER_ERROR_INVALID_VALUE,
+		   "Unknown internal child: %s", childname);
+    }
   return obj;
 }
 
 GObject *
 _gtk_builder_construct (GtkBuilder *builder,
-                        ObjectInfo *info)
+                        ObjectInfo *info,
+			GError **error)
 {
   GArray *parameters, *construct_parameters;
   GType object_type;
@@ -383,7 +380,14 @@ _gtk_builder_construct (GtkBuilder *builder,
   g_assert (info->class_name != NULL);
   object_type = gtk_builder_get_type_from_name (builder, info->class_name);
   if (object_type == G_TYPE_INVALID)
-    g_error ("Invalid type: %s", info->class_name);
+    {
+      g_set_error (error,
+		   GTK_BUILDER_ERROR,
+		   GTK_BUILDER_ERROR_INVALID_VALUE,
+		   "Invalid object type `%s'",
+		   info->class_name);
+      return NULL;
+    }
 
   gtk_builder_get_parameters (builder, object_type,
                               info->id,
@@ -397,23 +401,37 @@ _gtk_builder_construct (GtkBuilder *builder,
 
       constructor = gtk_builder_get_object (builder, info->constructor);
       if (constructor == NULL)
-        g_error ("Unknown constructor for %s: %s\n", info->id,
-                 info->constructor);
-
+	{
+	  g_set_error (error,
+		       GTK_BUILDER_ERROR,
+		       GTK_BUILDER_ERROR_INVALID_VALUE,
+		       "Unknown object constructor for %s: %s",
+		       info->id,
+		       info->constructor);
+	  g_array_free (parameters, TRUE);
+	  g_array_free (construct_parameters, TRUE);
+	  return NULL;
+	}
       obj = gtk_buildable_construct_child (GTK_BUILDABLE (constructor),
                                            builder,
                                            info->id);
       g_assert (obj != NULL);
       if (construct_parameters->len)
         g_warning ("Can't pass in construct-only parameters to %s", info->id);
-
     }
   else if (info->parent && ((ChildInfo*)info->parent)->internal_child != NULL)
     {
       gchar *childname = ((ChildInfo*)info->parent)->internal_child;
-      obj = gtk_builder_get_internal_child (builder, info, childname);
+      obj = gtk_builder_get_internal_child (builder, info, childname, error);
+      if (!obj)
+	{
+	  g_array_free (parameters, TRUE);
+	  g_array_free (construct_parameters, TRUE);
+	  return NULL;
+	}
       if (construct_parameters->len)
         g_warning ("Can't pass in construct-only parameters to %s", childname);
+      g_object_ref (obj);
     }
   else
     {
@@ -421,6 +439,18 @@ _gtk_builder_construct (GtkBuilder *builder,
                            construct_parameters->len,
                            (GParameter *)construct_parameters->data);
 
+      /* No matter what, make sure we have a reference.
+       *
+       * If it's an initially unowned object, sink it.
+       * If it's not initially unowned then we have the reference already.
+       *
+       * In the case that this is a window it will be sunk already and
+       * this is effectively a call to g_object_ref().  That's what
+       * we want.
+       */
+      if (G_IS_INITIALLY_UNOWNED (obj))
+        g_object_ref_sink (obj);
+
       GTK_NOTE (BUILDER,
                 g_print ("created %s of type %s\n", info->id, info->class_name));
 
@@ -472,15 +502,7 @@ _gtk_builder_construct (GtkBuilder *builder,
                             g_strdup (info->id),
                             g_free);
 
-
-  if (!info->parent && !GTK_IS_WINDOW (obj))
-    {
-      if (g_object_is_floating (obj))
-	  g_object_ref_sink (obj);
-      
-      builder->priv->root_objects =
-        g_slist_prepend (builder->priv->root_objects, obj);
-    }
+  /* we already own a reference to obj.  put it in the hash table. */
   g_hash_table_insert (builder->priv->objects, g_strdup (info->id), obj);
   
   return obj;
@@ -508,7 +530,7 @@ _gtk_builder_add (GtkBuilder *builder,
 
   if (!child_info->parent)
     {
-      g_warning ("%s: Not adding, No parent\n",
+      g_warning ("%s: Not adding, No parent",
                  gtk_buildable_get_name (GTK_BUILDABLE (object)));
       return;
     }
@@ -570,7 +592,7 @@ gtk_builder_apply_delayed_properties (GtkBuilder *builder)
       pspec = g_object_class_find_property (G_OBJECT_CLASS (oclass),
                                             property->name);
       if (!pspec)
-        g_warning ("Unknown property: %s.%s\n", g_type_name (object_type),
+        g_warning ("Unknown property: %s.%s", g_type_name (object_type),
                    property->name);
       else
         {
@@ -578,7 +600,7 @@ gtk_builder_apply_delayed_properties (GtkBuilder *builder)
 
           obj = g_hash_table_lookup (builder->priv->objects, property->value);
           if (!obj)
-            g_warning ("No object called: %s\n", property->value);
+            g_warning ("No object called: %s", property->value);
           else
             g_object_set (object, property->name, obj, NULL);
         }
@@ -636,6 +658,7 @@ gtk_builder_add_from_file (GtkBuilder   *builder,
 
   g_return_val_if_fail (GTK_IS_BUILDER (builder), 0);
   g_return_val_if_fail (filename != NULL, 0);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   tmp_error = NULL;
 
@@ -687,6 +710,7 @@ gtk_builder_add_from_string (GtkBuilder   *builder,
 
   g_return_val_if_fail (GTK_IS_BUILDER (builder), 0);
   g_return_val_if_fail (buffer != NULL, 0);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   tmp_error = NULL;
 
@@ -850,6 +874,12 @@ gtk_builder_connect_signals_default (GtkBuilder    *builder,
  * Note that this function will not work correctly if #GModule is not
  * supported on the platform.
  *
+ * When compiling applications for Windows, you must declare signal callbacks
+ * with #G_MODULE_EXPORT, or they will not be put in the symbol table.
+ * On Linux and Unices, this is not necessary; applications should instead
+ * be compiled with the -Wl,--export-dynamic CFLAGS, and linked against
+ * gmodule-export-2.0.
+ *
  * Since: 2.12
  **/
 void
@@ -982,6 +1012,12 @@ gtk_builder_value_from_string (GtkBuilder   *builder,
                                GValue       *value,
 			       GError      **error)
 {
+  g_return_val_if_fail (GTK_IS_BUILDER (builder), FALSE);
+  g_return_val_if_fail (G_IS_PARAM_SPEC (pspec), FALSE);
+  g_return_val_if_fail (string != NULL, FALSE);
+  g_return_val_if_fail (value != NULL, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
   /*
    * GParamSpecUnichar has the internal type G_TYPE_UINT,
    * so we cannot handle this in the switch, do it separately
@@ -1029,6 +1065,7 @@ gtk_builder_value_from_string_type (GtkBuilder   *builder,
 
   g_return_val_if_fail (type != G_TYPE_INVALID, FALSE);
   g_return_val_if_fail (string != NULL, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   g_value_init (value, type);
 
@@ -1111,7 +1148,8 @@ gtk_builder_value_from_string_type (GtkBuilder   *builder,
       }
     case G_TYPE_FLAGS:
       {
-	gint flags_value;
+	guint flags_value;
+
 	if (!_gtk_builder_flags_from_string (type, string, &flags_value, error))
 	  {
 	    ret = FALSE;
@@ -1191,18 +1229,7 @@ gtk_builder_value_from_string_type (GtkBuilder   *builder,
               return FALSE;
             }
 
-          if (g_path_is_absolute (string))
-            filename = g_strdup (string);
-          else
-            {
-              gchar *dirname;
-
-              dirname = g_path_get_dirname (builder->priv->filename);
-              filename = g_build_filename (dirname, string, NULL);
-
-              g_free (dirname);
-            }
-
+	  filename = _gtk_builder_get_absolute_filename (builder, string);
           pixbuf = gdk_pixbuf_new_from_file (filename, &tmp_error);
 
           if (pixbuf == NULL)
@@ -1248,7 +1275,7 @@ gtk_builder_value_from_string_type (GtkBuilder   *builder,
   return ret;
 }
 
-static gboolean
+gboolean
 _gtk_builder_enum_from_string (GType         type, 
                                const gchar  *string,
 			       gint         *enum_value,
@@ -1258,10 +1285,13 @@ _gtk_builder_enum_from_string (GType         type,
   GEnumValue *ev;
   gchar *endptr;
   gint value;
+  gboolean ret;
   
-  g_return_val_if_fail (G_TYPE_IS_ENUM (type), 0);
-  g_return_val_if_fail (string != NULL, 0);
+  g_return_val_if_fail (G_TYPE_IS_ENUM (type), FALSE);
+  g_return_val_if_fail (string != NULL, FALSE);
   
+  ret = TRUE;
+
   value = strtoul (string, &endptr, 0);
   if (endptr != string) /* parsed a number */
     *enum_value = value;
@@ -1281,32 +1311,32 @@ _gtk_builder_enum_from_string (GType         type,
 		       GTK_BUILDER_ERROR_INVALID_VALUE,
 		       "Could not parse enum: `%s'",
 		       string);
-	  return FALSE;
+	  ret = FALSE;
 	}
       
       g_type_class_unref (eclass);
     }
   
-  return TRUE;
+  return ret;
 }
 
 gboolean
 _gtk_builder_flags_from_string (GType         type, 
                                 const gchar  *string,
-				gint         *flags_value,
+				guint        *flags_value,
 				GError      **error)
 {
   GFlagsClass *fclass;
   gchar *endptr, *prevptr;
-  guint i, j, ret, value;
+  guint i, j, value;
   gchar *flagstr;
   GFlagsValue *fv;
   const gchar *flag;
   gunichar ch;
-  gboolean eos;
+  gboolean eos, ret;
 
-  g_return_val_if_fail (G_TYPE_IS_FLAGS (type), 0);
-  g_return_val_if_fail (string != 0, 0);
+  g_return_val_if_fail (G_TYPE_IS_FLAGS (type), FALSE);
+  g_return_val_if_fail (string != 0, FALSE);
 
   ret = TRUE;
   
@@ -1420,6 +1450,34 @@ gtk_builder_error_quark (void)
   return g_quark_from_static_string ("gtk-builder-error-quark");
 }
 
+gchar *
+_gtk_builder_get_absolute_filename (GtkBuilder *builder, const gchar *string)
+{
+  gchar *filename;
+  gchar *dirname = NULL;
+  
+  if (g_path_is_absolute (string))
+    return g_strdup (string);
+
+  if (builder->priv->filename &&
+      strcmp (builder->priv->filename, ".") != 0) 
+    {
+      dirname = g_path_get_dirname (builder->priv->filename);
+
+      if (strcmp (dirname, ".") == 0)
+	{
+	  g_free (dirname);
+	  dirname = g_get_current_dir ();
+	}
+    }
+  else
+    dirname = g_get_current_dir ();
+    
+  filename = g_build_filename (dirname, string, NULL);
+  g_free (dirname);
+  
+  return filename;
+}
 
 #define __GTK_BUILDER_C__
 #include "gtkaliasdef.c"
