@@ -22,10 +22,11 @@
  */
 
 #include <config.h>
-
+#include <string.h>
 #include "gtkexpander.h"
 
 #include "gtklabel.h"
+#include "gtkbuildable.h"
 #include "gtkcontainer.h"
 #include "gtkmarshalers.h"
 #include "gtkmain.h"
@@ -126,7 +127,16 @@ static void gtk_expander_activate (GtkExpander *expander);
 static void get_expander_bounds (GtkExpander  *expander,
 				 GdkRectangle *rect);
 
-G_DEFINE_TYPE (GtkExpander, gtk_expander, GTK_TYPE_BIN)
+/* GtkBuildable */
+static void gtk_expander_buildable_init           (GtkBuildableIface *iface);
+static void gtk_expander_buildable_add_child      (GtkBuildable *buildable,
+						   GtkBuilder   *builder,
+						   GObject      *child,
+						   const gchar  *type);
+
+G_DEFINE_TYPE_WITH_CODE (GtkExpander, gtk_expander, GTK_TYPE_BIN,
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
+						gtk_expander_buildable_init))
 
 static void
 gtk_expander_class_init (GtkExpanderClass *klass)
@@ -277,6 +287,26 @@ gtk_expander_init (GtkExpander *expander)
   gtk_drag_dest_set_track_motion (GTK_WIDGET (expander), TRUE);
 }
 
+static void
+gtk_expander_buildable_add_child (GtkBuildable  *buildable,
+				  GtkBuilder    *builder,
+				  GObject       *child,
+				  const gchar   *type)
+{
+  if (!type)
+    gtk_container_add (GTK_CONTAINER (buildable), GTK_WIDGET (child));
+  else if (strcmp (type, "label") == 0)
+    gtk_expander_set_label_widget (GTK_EXPANDER (buildable), GTK_WIDGET (child));
+  else
+    GTK_BUILDER_WARN_INVALID_CHILD_TYPE (GTK_EXPANDER (buildable), type);
+}
+
+static void
+gtk_expander_buildable_init (GtkBuildableIface *iface)
+{
+  iface->add_child = gtk_expander_buildable_add_child;
+}
+
 static void
 gtk_expander_set_property (GObject      *object,
 			   guint         prop_id,
@@ -411,7 +441,6 @@ gtk_expander_realize (GtkWidget *widget)
   gdk_window_set_user_data (priv->event_window, widget);
 
   widget->style = gtk_style_attach (widget->style, widget->window);
-  //gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
 }
 
 static void
@@ -1017,7 +1046,7 @@ gtk_expander_drag_motion (GtkWidget        *widget,
       settings = gtk_widget_get_settings (widget);
       g_object_get (settings, "gtk-timeout-expand", &timeout, NULL);
 
-      priv->expand_timer = g_timeout_add (timeout, (GSourceFunc) expand_timeout, expander);
+      priv->expand_timer = gdk_threads_add_timeout (timeout, (GSourceFunc) expand_timeout, expander);
     }
 
   return TRUE;
@@ -1289,8 +1318,6 @@ gtk_expander_animation_timeout (GtkExpander *expander)
   GdkRectangle area;
   gboolean finish = FALSE;
 
-  GDK_THREADS_ENTER();
-
   if (GTK_WIDGET_REALIZED (expander))
     {
       get_expander_bounds (expander, &area);
@@ -1330,8 +1357,6 @@ gtk_expander_animation_timeout (GtkExpander *expander)
       gtk_widget_queue_resize (GTK_WIDGET (expander));
     }
 
-  GDK_THREADS_LEAVE();
-
   return !finish;
 }
 
@@ -1344,7 +1369,7 @@ gtk_expander_start_animation (GtkExpander *expander)
     g_source_remove (priv->animation_timeout);
 
   priv->animation_timeout =
-		g_timeout_add (50,
+		gdk_threads_add_timeout (50,
 			       (GSourceFunc) gtk_expander_animation_timeout,
 			       expander);
 }
@@ -1507,11 +1532,11 @@ gtk_expander_set_label (GtkExpander *expander,
  * gtk_expander_get_label:
  * @expander: a #GtkExpander
  *
- * Fetches the text from the label of the expander, as set by
- * gtk_expander_set_label(). If the label text has not
- * been set the return value will be %NULL. This will be the
- * case if you create an empty button with gtk_button_new() to
- * use as a container.
+ * Fetches the text from a label widget including any embedded
+ * underlines indicating mnemonics and Pango markup, as set by
+ * gtk_expander_set_label(). If the label text has not been set the
+ * return value will be %NULL. This will be the case if you create an
+ * empty button with gtk_button_new() to use as a container.
  *
  * Return value: The text of the label widget. This string is owned
  * by the widget and must not be modified or freed.
@@ -1527,8 +1552,8 @@ gtk_expander_get_label (GtkExpander *expander)
 
   priv = expander->priv;
 
-  if (priv->label_widget && GTK_IS_LABEL (priv->label_widget))
-    return gtk_label_get_text (GTK_LABEL (priv->label_widget));
+  if (GTK_IS_LABEL (priv->label_widget))
+    return gtk_label_get_label (GTK_LABEL (priv->label_widget));
   else
     return NULL;
 }
@@ -1559,7 +1584,7 @@ gtk_expander_set_use_underline (GtkExpander *expander,
     {
       priv->use_underline = use_underline;
 
-      if (priv->label_widget && GTK_IS_LABEL (priv->label_widget))
+      if (GTK_IS_LABEL (priv->label_widget))
 	gtk_label_set_use_underline (GTK_LABEL (priv->label_widget), use_underline);
 
       g_object_notify (G_OBJECT (expander), "use-underline");
@@ -1613,7 +1638,7 @@ gtk_expander_set_use_markup (GtkExpander *expander,
     {
       priv->use_markup = use_markup;
 
-      if (priv->label_widget && GTK_IS_LABEL (priv->label_widget))
+      if (GTK_IS_LABEL (priv->label_widget))
 	gtk_label_set_use_markup (GTK_LABEL (priv->label_widget), use_markup);
 
       g_object_notify (G_OBJECT (expander), "use-markup");
