@@ -18,11 +18,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_FILE_CHOOSER_DIALOG_H__
 #define __GTK_FILE_CHOOSER_DIALOG_H__
 
 #include <gtk/gtkdialog.h>
-#include "gtkfilechooser.h"
+#include <gtk/gtkfilechooser.h>
 
 G_BEGIN_DECLS
 
@@ -33,15 +37,9 @@ G_BEGIN_DECLS
 #define GTK_IS_FILE_CHOOSER_DIALOG_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_FILE_CHOOSER_DIALOG))
 #define GTK_FILE_CHOOSER_DIALOG_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_FILE_CHOOSER_DIALOG, GtkFileChooserDialogClass))
 
-typedef struct _GtkFileChooserDialog      GtkFileChooserDialog;
-typedef struct _GtkFileChooserDialogClass GtkFileChooserDialogClass;
-
+typedef struct _GtkFileChooserDialog        GtkFileChooserDialog;
 typedef struct _GtkFileChooserDialogPrivate GtkFileChooserDialogPrivate;
-
-struct _GtkFileChooserDialogClass
-{
-  GtkDialogClass parent_class;
-};
+typedef struct _GtkFileChooserDialogClass   GtkFileChooserDialogClass;
 
 struct _GtkFileChooserDialog
 {
@@ -50,18 +48,23 @@ struct _GtkFileChooserDialog
   GtkFileChooserDialogPrivate *priv;
 };
 
-GType      gtk_file_chooser_dialog_get_type         (void);
+struct _GtkFileChooserDialogClass
+{
+  GtkDialogClass parent_class;
+};
+
+GType      gtk_file_chooser_dialog_get_type         (void) G_GNUC_CONST;
 GtkWidget *gtk_file_chooser_dialog_new              (const gchar          *title,
 						     GtkWindow            *parent,
 						     GtkFileChooserAction  action,
 						     const gchar          *first_button_text,
-						     ...);
+						     ...) G_GNUC_NULL_TERMINATED;
 GtkWidget *gtk_file_chooser_dialog_new_with_backend (const gchar          *title,
 						     GtkWindow            *parent,
 						     GtkFileChooserAction  action,
 						     const gchar          *backend,
 						     const gchar          *first_button_text,
-						     ...);
+						     ...) G_GNUC_NULL_TERMINATED;
 
 G_END_DECLS
 
