@@ -32,7 +32,7 @@
 #include <config.h>
 #include <errno.h>
 #include <string.h>
-#include <glib/gkeyfile.h>
+#include <glib.h>
 #include <glib/gi18n-lib.h>
 #include "gtkfilechoosersettings.h"
 #include "gtkalias.h"
