@@ -114,7 +114,7 @@ gtk_frame_class_init (GtkFrameClass *class)
 						       P_("The horizontal alignment of the label"),
 						       0.0,
 						       1.0,
-						       0.5,
+						       0.0,
 						       GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
 				   PROP_LABEL_YALIGN,
@@ -349,7 +349,7 @@ gtk_frame_get_label (GtkFrame *frame)
 {
   g_return_val_if_fail (GTK_IS_FRAME (frame), NULL);
 
-  if (frame->label_widget && GTK_IS_LABEL (frame->label_widget))
+  if (GTK_IS_LABEL (frame->label_widget))
     return gtk_label_get_text (GTK_LABEL (frame->label_widget));
   else
     return NULL;
@@ -676,7 +676,7 @@ gtk_frame_size_allocate (GtkWidget     *widget,
       
       child_allocation.x = frame->child_allocation.x + LABEL_SIDE_PAD +
 	(frame->child_allocation.width - child_requisition.width - 2 * LABEL_PAD - 2 * LABEL_SIDE_PAD) * xalign + LABEL_PAD;
-      child_allocation.width = child_requisition.width;
+      child_allocation.width = MIN (child_requisition.width, new_allocation.width - 2 * LABEL_PAD - 2 * LABEL_SIDE_PAD);
 
       child_allocation.y = frame->child_allocation.y - MAX (child_requisition.height, widget->style->ythickness);
       child_allocation.height = child_requisition.height;
