@@ -842,7 +842,6 @@ gtk_hsv_motion (GtkWidget      *widget,
   GtkHSV *hsv;
   HSVPrivate *priv;
   double x, y;
-  gint ix, iy;
   GdkModifierType mods;
   
   hsv = GTK_HSV (widget);
@@ -851,18 +850,11 @@ gtk_hsv_motion (GtkWidget      *widget,
   if (priv->mode == DRAG_NONE)
     return FALSE;
   
-  if (event->is_hint)
-    {
-      gdk_window_get_pointer (priv->window, &ix, &iy, &mods);
-      x = ix;
-      y = iy;
-    }
-  else
-    {
-      x = event->x;
-      y = event->y;
-    }
-  
+  gdk_event_request_motions (event);
+  x = event->x;
+  y = event->y;
+  mods = event->state;
+
   if (priv->mode == DRAG_H)
     {
       gtk_hsv_set_color (hsv, compute_v (hsv, x, y), priv->s, priv->v);
@@ -905,6 +897,7 @@ paint_ring (GtkHSV      *hsv,
   gdouble r, g, b;
   cairo_surface_t *source;
   cairo_t *source_cr;
+  gint stride;
   gint focus_width;
   gint focus_pad;
 
@@ -922,7 +915,8 @@ paint_ring (GtkHSV      *hsv,
   
   /* Create an image initialized with the ring colors */
   
-  buf = g_new (guint32, width * height);
+  stride = cairo_format_stride_for_width (CAIRO_FORMAT_RGB24, width);
+  buf = g_new (guint32, height * stride / 4);
   
   for (yy = 0; yy < height; yy++)
     {
@@ -960,7 +954,7 @@ paint_ring (GtkHSV      *hsv,
 
   source = cairo_image_surface_create_for_data ((char *)buf,
 						CAIRO_FORMAT_RGB24,
-						width, height, 4 * width);
+						width, height, stride);
 
   /* Now draw the value marker onto the source image, so that it
    * will get properly clipped at the edges of the ring
@@ -1055,6 +1049,7 @@ paint_triangle (GtkHSV      *hsv,
   cairo_surface_t *source;
   gdouble r, g, b;
   gchar *detail;
+  gint stride;
   
   priv = hsv->priv;
   
@@ -1102,8 +1097,9 @@ paint_triangle (GtkHSV      *hsv,
     }
   
   /* Shade the triangle */
-  
-  buf = g_new (guint32, width * height);
+
+  stride = cairo_format_stride_for_width (CAIRO_FORMAT_RGB24, width);
+  buf = g_new (guint32, height * stride / 4);
   
   for (yy = 0; yy < height; yy++)
     {
@@ -1170,7 +1166,7 @@ paint_triangle (GtkHSV      *hsv,
 
   source = cairo_image_surface_create_for_data ((char *)buf,
 						CAIRO_FORMAT_RGB24,
-						width, height, 4 * width);
+						width, height, stride);
   
   /* Draw a triangle with the image as a source */
 
