@@ -21,7 +21,7 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
 #include <config.h>
@@ -64,7 +64,7 @@ static void gtk_image_menu_item_screen_changed (GtkWidget        *widget,
 
 
 enum {
-  PROP_ZERO,
+  PROP_0,
   PROP_IMAGE
 };
 
@@ -616,7 +616,7 @@ gtk_image_menu_item_screen_changed (GtkWidget *widget,
 
   show_image_connection =
     g_signal_connect (settings, "notify::gtk-menu-images",
-		      G_CALLBACK (gtk_image_menu_item_setting_changed), 0);
+		      G_CALLBACK (gtk_image_menu_item_setting_changed), NULL);
   g_object_set_data (G_OBJECT (settings), 
 		     I_("gtk-image-menu-item-connection"),
 		     GUINT_TO_POINTER (show_image_connection));
