@@ -31,7 +31,6 @@
 #include "gtkobject.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
-#include "gtksignal.h"
 #include "gtkprivate.h"
 #include "gtkalias.h"
 
@@ -82,7 +81,7 @@ gtk_object_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
 	sizeof (GtkObjectClass),
 	(GBaseInitFunc) gtk_object_base_class_init,
