@@ -22,18 +22,19 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_PANED_H__
 #define __GTK_PANED_H__
 
 #include <gtk/gtkcontainer.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GTK_TYPE_PANED                  (gtk_paned_get_type ())
 #define GTK_PANED(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_PANED, GtkPaned))
@@ -43,20 +44,21 @@ extern "C" {
 #define GTK_PANED_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_PANED, GtkPanedClass))
 
 
-typedef struct _GtkPaned      GtkPaned;
-typedef struct _GtkPanedClass GtkPanedClass;
+typedef struct _GtkPaned        GtkPaned;
+typedef struct _GtkPanedClass   GtkPanedClass;
+typedef struct _GtkPanedPrivate GtkPanedPrivate;
 
 struct _GtkPaned
 {
   GtkContainer container;
-  
+
   GtkWidget *child1;
   GtkWidget *child2;
-  
+
   GdkWindow *handle;
   GdkGC *xor_gc;
   GdkCursorType cursor_type;
-  
+
   /*< private >*/
   GdkRectangle handle_pos;
 
@@ -77,18 +79,18 @@ struct _GtkPaned
 
   GtkWidget *last_child1_focus;
   GtkWidget *last_child2_focus;
-  GtkWidget *saved_focus;
+  GtkPanedPrivate *priv;
 
   gint drag_pos;
   gint original_position;
 };
- 
+
 struct _GtkPanedClass
 {
   GtkContainerClass parent_class;
-  
+
   gboolean (* cycle_child_focus)   (GtkPaned      *paned,
-				    gboolean       reverse); 
+				    gboolean       reverse);
   gboolean (* toggle_handle_focus) (GtkPaned      *paned);
   gboolean (* move_handle)         (GtkPaned      *paned,
 				    GtkScrollType  scroll);
@@ -122,19 +124,21 @@ gint    gtk_paned_get_position    (GtkPaned  *paned);
 void    gtk_paned_set_position    (GtkPaned  *paned,
 				   gint       position);
 
+GtkWidget *gtk_paned_get_child1   (GtkPaned  *paned);
+GtkWidget *gtk_paned_get_child2   (GtkPaned  *paned);
+
 /* Internal function */
+#if !defined (GTK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
 void    gtk_paned_compute_position (GtkPaned  *paned,
 				    gint       allocation,
 				    gint       child1_req,
 				    gint       child2_req);
-
+#endif /* !GTK_DISABLE_DEPRECATED || GTK_COMPILATION */
 #ifndef GTK_DISABLE_DEPRECATED
 #define	gtk_paned_gutter_size(p,s)		(void) 0
 #define	gtk_paned_set_gutter_size(p,s)		(void) 0
 #endif /* GTK_DISABLE_DEPRECATED */
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GTK_PANED_H__ */
