@@ -355,7 +355,7 @@ gtk_radio_menu_item_class_init (GtkRadioMenuItemClass *klass)
    * Emitted when the group of radio menu items that a radio menu item belongs
    * to changes. This is emitted when a radio menu item switches from
    * being alone to being part of a group of 2 or more menu items, or
-   * vice-versa, and when a buttton is moved from one group of 2 or
+   * vice-versa, and when a button is moved from one group of 2 or
    * more menu items ton a different one, but not when the composition
    * of the group that a menu item belongs to changes.
    *
@@ -380,16 +380,12 @@ gtk_radio_menu_item_init (GtkRadioMenuItem *radio_menu_item)
 static void
 gtk_radio_menu_item_destroy (GtkObject *object)
 {
+  GtkRadioMenuItem *radio_menu_item = GTK_RADIO_MENU_ITEM (object);
   GtkWidget *old_group_singleton = NULL;
-  GtkRadioMenuItem *radio_menu_item;
   GtkRadioMenuItem *tmp_menu_item;
   GSList *tmp_list;
   gboolean was_in_group;
 
-  g_return_if_fail (GTK_IS_RADIO_MENU_ITEM (object));
-
-  radio_menu_item = GTK_RADIO_MENU_ITEM (object);
-
   was_in_group = radio_menu_item->group && radio_menu_item->group->next;
   
   radio_menu_item->group = g_slist_remove (radio_menu_item->group,
@@ -422,16 +418,12 @@ gtk_radio_menu_item_destroy (GtkObject *object)
 static void
 gtk_radio_menu_item_activate (GtkMenuItem *menu_item)
 {
-  GtkRadioMenuItem *radio_menu_item;
-  GtkCheckMenuItem *check_menu_item;
+  GtkRadioMenuItem *radio_menu_item = GTK_RADIO_MENU_ITEM (menu_item);
+  GtkCheckMenuItem *check_menu_item = GTK_CHECK_MENU_ITEM (menu_item);
   GtkCheckMenuItem *tmp_menu_item;
   GSList *tmp_list;
   gint toggled;
 
-  g_return_if_fail (GTK_IS_RADIO_MENU_ITEM (menu_item));
-
-  radio_menu_item = GTK_RADIO_MENU_ITEM (menu_item);
-  check_menu_item = GTK_CHECK_MENU_ITEM (menu_item);
   toggled = FALSE;
 
   if (check_menu_item->active)
