@@ -21,7 +21,7 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
 /*
@@ -34,6 +34,10 @@
  * distribution.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_RULER_H__
 #define __GTK_RULER_H__
 
@@ -64,7 +68,7 @@ struct _GtkRuler
   GtkWidget widget;
 
   GdkPixmap *backing_store;
-  GdkGC *non_gr_exp_gc;
+  GdkGC *non_gr_exp_gc;		/* unused */
   GtkRulerMetric *metric;
   gint xsrc, ysrc;
   gint slider_size;
