@@ -22,9 +22,13 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_STATUSBAR_H__
 #define __GTK_STATUSBAR_H__
 
@@ -57,7 +61,7 @@ struct _GtkStatusbar
   guint seq_message_id;
 
   GdkWindow *grip_window;
-  
+
   guint has_resize_grip : 1;
 };
 
