@@ -630,9 +630,10 @@ gtk_tree_model_sort_row_inserted (GtkTreeModel          *s_model,
 
       if (level->array->len < gtk_tree_path_get_indices (s_path)[i])
 	{
-	  g_warning ("A node was inserted with a parent that's not in the tree.\n"
+	  g_warning ("%s: A node was inserted with a parent that's not in the tree.\n"
 		     "This possibly means that a GtkTreeModel inserted a child node\n"
-		     "before the parent was inserted.");
+		     "before the parent was inserted.",
+		     G_STRLOC);
 	  goto done;
 	}
 
@@ -2059,31 +2060,44 @@ gtk_tree_model_sort_convert_child_path_to_path (GtkTreeModelSort *tree_model_sor
  * @child_iter: A valid #GtkTreeIter pointing to a row on the child model
  * 
  * Sets @sort_iter to point to the row in @tree_model_sort that corresponds to
- * the row pointed at by @child_iter.
+ * the row pointed at by @child_iter.  If @sort_iter was not set, %FALSE
+ * is returned.  Note: a boolean is only returned since 2.14.
+ *
+ * Return value: %TRUE, if @sort_iter was set, i.e. if @sort_iter is a
+ * valid iterator pointer to a visible row in the child model.
  **/
-void
+gboolean
 gtk_tree_model_sort_convert_child_iter_to_iter (GtkTreeModelSort *tree_model_sort,
 						GtkTreeIter      *sort_iter,
 						GtkTreeIter      *child_iter)
 {
+  gboolean ret;
   GtkTreePath *child_path, *path;
 
-  g_return_if_fail (GTK_IS_TREE_MODEL_SORT (tree_model_sort));
-  g_return_if_fail (tree_model_sort->child_model != NULL);
-  g_return_if_fail (sort_iter != NULL);
-  g_return_if_fail (child_iter != NULL);
+  g_return_val_if_fail (GTK_IS_TREE_MODEL_SORT (tree_model_sort), FALSE);
+  g_return_val_if_fail (tree_model_sort->child_model != NULL, FALSE);
+  g_return_val_if_fail (sort_iter != NULL, FALSE);
+  g_return_val_if_fail (child_iter != NULL, FALSE);
 
   sort_iter->stamp = 0;
 
   child_path = gtk_tree_model_get_path (tree_model_sort->child_model, child_iter);
-  g_return_if_fail (child_path != NULL);
+  g_return_val_if_fail (child_path != NULL, FALSE);
 
   path = gtk_tree_model_sort_convert_child_path_to_path (tree_model_sort, child_path);
   gtk_tree_path_free (child_path);
-  g_return_if_fail (path != NULL);
 
-  gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_model_sort), sort_iter, path);
+  if (!path)
+    {
+      g_warning ("%s: The conversion of the child path to a GtkTreeModel sort path failed", G_STRLOC);
+      return FALSE;
+    }
+
+  ret = gtk_tree_model_get_iter (GTK_TREE_MODEL (tree_model_sort),
+                                 sort_iter, path);
   gtk_tree_path_free (path);
+
+  return ret;
 }
 
 /**
@@ -2259,7 +2273,30 @@ gtk_tree_model_sort_build_level (GtkTreeModelSort *tree_model_sort,
 	  if (gtk_tree_model_iter_next (tree_model_sort->child_model, &iter) == FALSE &&
 	      i < length - 1)
 	    {
-	      g_warning ("There is a discrepancy between the sort model and the child model.");
+	      if (parent_level)
+	        {
+	          GtkTreePath *level;
+		  gchar *str;
+
+		  level = gtk_tree_model_sort_elt_get_path (parent_level,
+							    parent_elt);
+		  str = gtk_tree_path_to_string (level);
+		  gtk_tree_path_free (level);
+
+		  g_warning ("%s: There is a discrepancy between the sort model "
+			     "and the child model.  The child model is "
+			     "advertising a wrong length for level %s:.",
+			     G_STRLOC, str);
+		  g_free (str);
+		}
+	      else
+	        {
+		  g_warning ("%s: There is a discrepancy between the sort model "
+			     "and the child model.  The child model is "
+			     "advertising a wrong length for the root level.",
+			     G_STRLOC);
+		}
+
 	      return;
 	    }
 	}
