@@ -25,14 +25,15 @@
  *
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "gtk/gtkintl.h"
-#include "gtk/gtkimcontextsimple.h"
 #include "gtk/gtkimmodule.h"
+#include "gtk/gtkintl.h"
 
 GType type_am_et_translit = 0;
 
@@ -460,30 +461,32 @@ static const GtkIMContextInfo *info_list[] = {
   &am_et_info
 };
 
-void
-im_module_init (GTypeModule *module)
+#ifndef INCLUDE_IM_am_et
+#define MODULE_ENTRY(type,function) G_MODULE_EXPORT type im_module_ ## function
+#else
+#define MODULE_ENTRY(type, function) type _gtk_immodule_am_et_ ## function
+#endif
+
+MODULE_ENTRY (void, init) (GTypeModule *module)
 {
   am_et_register_type (module);
 }
 
-void 
-im_module_exit (void)
+MODULE_ENTRY (void, exit) (void)
 {
 }
 
-void 
-im_module_list (const GtkIMContextInfo ***contexts,
-		int                      *n_contexts)
+MODULE_ENTRY (void, list) (const GtkIMContextInfo ***contexts,
+			   int                      *n_contexts)
 {
   *contexts = info_list;
   *n_contexts = G_N_ELEMENTS (info_list);
 }
 
-GtkIMContext *
-im_module_create (const gchar *context_id)
+MODULE_ENTRY (GtkIMContext *, create) (const gchar *context_id)
 {
   if (strcmp (context_id, "am_et") == 0)
-    return GTK_IM_CONTEXT (g_object_new (type_am_et_translit,NULL));
+    return g_object_new (type_am_et_translit, NULL);
   else
     return NULL;
 }
