@@ -496,6 +496,7 @@ gail_text_view_set_caret_offset (AtkText *text,
 
   gtk_text_buffer_get_iter_at_offset (buffer,  &pos_itr, offset);
   gtk_text_buffer_place_cursor (buffer, &pos_itr);
+  gtk_text_view_scroll_to_iter (view, &pos_itr, 0, FALSE, 0, 0);
   return TRUE;
 }
 
@@ -1736,7 +1737,7 @@ static GIOChannel*       gail_streamable_content_get_stream       (AtkStreamable
 	    else g_message (err->message);
 	    if (err) {
 		g_message ("<error writing to stream [%s]>", tname);
-		g_free (err);
+		g_error_free (err);
 	    }
 	    /* make sure the file is removed on unref of the giochannel */
 	    else {
